#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import logging
import time
import datetime
import calendar
import locale
import json
import codecs
import shutil
import sys

sys.path += [os.path.join(os.path.dirname(__file__), '..')]

import reportTemplateDef
from report_util       import util
from reportTemplateDef import chartType
from report_config     import ReportConfig
from report_engine     import ReportEngine

currentDir = os.path.dirname(os.path.abspath(__file__))

class ChartDataHandler():
    htmlJumper = u'<div class="jump"><a href="#top">Back to Top</a></div>'
    def __init__(self, params, info, reportData):
        logging.debug('ChartDataHandler.__init__')
        self.params         = params
        self.info           = info
        self.reportData     = reportData
        self.chartData      = []
        self.id             = 0
        self.lineTimePoints = []
        self.lineTimeszSet  = []
        self.htmlContainer  = ''
        self.htmlSelector   = ''
        if not os.path.exists(ReportConfig.REPORT_TEMP_LOCATION):
            os.makedirs(ReportConfig.REPORT_TEMP_LOCATION)

    def buildChartData(self):
        logging.debug('ChartDataHandler.buildChartData')

        data = None
        for item in self.params:
            if item.custom:
                #generate facet table
                data = self.generateTableData(item.favID, item.displayName, True)
                if data:
                    self.chartData.append(data)

                #generate chart
                if item.chartType == chartType['line']:
                    #time chart
                    data = self.generateLineChartData('/'.join([item.favID, 'timeChart']), '', True)
                else:
                    if item.viewChart:
                        reportNm = '/'.join([item.favID, item.viewChart])
                        if self.reportData['custom_reports'][item.favID] and self.reportData['custom_reports'][item.favID].has_key(item.viewChart):
                            self.reportData['custom_reports'][reportNm] = self.reportData['custom_reports'][item.favID][item.viewChart]
                        else:
                            self.reportData['custom_reports'][reportNm] = []
                        if item.chartType == chartType['bar']:
                            data = self.generateBarChartData(reportNm, '', True)
                        elif item.chartType == chartType['pie']:
                            total = self.reportData['custom_reports']['/'.join((item.favID, 'total'))]
                            data = self.generatePieChartData(reportNm, '', total, True)
                if data:
                    self.chartData.append(data)

                #get raw log
                rawLogs = ReportEngine.getRawLog(item.reportName, self.info.timeFrom, self.info.timeTo, item.filters, item.columns, self.info.timeType)
                self.reportData['custom_reports']['/'.join((item.favID, 'rawLogs'))] = rawLogs
                data = self.generateTableData2(rawLogs, '')
                if data:
                    self.chartData.append(data)

            else:
                if item.chartType == chartType['bar']:
                    data = self.generateBarChartData(item.reportName, item.displayName)
                elif item.chartType == chartType['line']:
                    data = self.generateLineChartData(item.reportName, item.displayName)
                elif item.chartType == chartType['table']:
                    data = self.generateTableData(item.reportName, item.displayName)
                elif item.chartType == chartType['pie']:
                    data = self.generatePieChartData(item.reportName, item.displayName)

                if data:
                    self.chartData.append(data)

    @staticmethod
    def _convertNumberToStr(num):
        if num < 1000:
            return str(num)
        elif num < 1000 * 1000:
            return str(num / 1000.0) + 'K'
        else:
            return str(num / (1000000.0)) +'M'

    def generateTableData(self, reportName, displayName, custom=False):
        logging.info('ChartDataHandler.generateTableData')

        if custom:
            if reportName not in self.reportData['custom_reports']:
                return
            else:
                report  = self.reportData['custom_reports'][reportName]
        else:
            if reportName not in self.reportData:
                return
            else:
                report  = self.reportData[reportName]

        catSet  = []
        dataSet = []

        if not report:
            report = []

        translate_fields = False
        fields_i18n = None
        if isinstance(report, list):
            for name,value in report:
                catSet.append(name)
                dataSet.append(value)
        elif isinstance(report, dict):
            if report.has_key('CLF_Fields'):
                translate_fields = True
                fields_i18n = report['CLF_Fields']
            for key in report:
                if key == 'CLF_Fields':
                    continue

                if translate_fields:
                    catSet.append(fields_i18n[key])
                else:
                    catSet.append(key)
                dataSet.append('')
                for name,value in report[key]:
                    catSet.append(name)
                    dataSet.append(value)

        if displayName:
            self._createSelector(displayName)
        location  = u'''<a name="loc%d" ></a>''' % (self.id)

        container = u'container%d' % (self.id)
        htmlPiece = u'<div id="%s" style="width: 100%%; text-align:center">' % (container)

        htmlPiece += u'''<table width="100%" border="1">'''
        htmlPiece += u'\n'
        htmlPiece += u'<caption>' + displayName + '</caption>'
        htmlPiece += u'\n'
        htmlPiece += u'<tbody>'
        htmlPiece += u'\n'

        for i in range(0,len(catSet)):
            if (dataSet[i] == ''):
                htmlPiece += u'<tr><td colspan=2 align="center">'
                htmlPiece += catSet[i]
                htmlPiece += u'</td></tr>'
            else:
                htmlPiece += u'<tr><td>'
                htmlPiece += catSet[i]
                htmlPiece += u'</td><td>'
                htmlPiece += str(dataSet[i])
                htmlPiece += u'</td></tr>'
        htmlPiece += u'\n'
        htmlPiece += u'</tbody>'
        htmlPiece += u'\n'
        htmlPiece += u'</table>'
        htmlPiece += u'</div>'

        if self.htmlContainer:
            self.htmlContainer += u'\n'
        self.htmlContainer += location
        self.htmlContainer += u'\n'
        self.htmlContainer += ChartDataHandler.htmlJumper
        self.htmlContainer += u'\n'
        self.htmlContainer += htmlPiece

        self.id += 1

    def generateTableData2(self, data, displayName, headers = ''):
        logging.info('ChartDataHandler.generateTableData2')

        if displayName:
            self._createSelector(displayName)
        location  = u'''<a name="loc%d" ></a>''' % (self.id)

        container = u'container%d' % (self.id)
        htmlPiece = u'<div id="%s" style="width: 100%%; text-align:center">' % (container)

        htmlPiece += u'''<table width="100%" border="1">'''
        htmlPiece += u'\n'
        htmlPiece += u'<caption>' + displayName + '</caption>'
        htmlPiece += u'\n'
        htmlPiece += u'<tbody>'
        htmlPiece += u'\n'

        #headers
        if headers:
            htmlPiece += u'<tr>'
            for fl in headers:
                htmlPiece += u'<td>'
                htmlPiece += str(fl)
                htmlPiece += u'</td>'
            htmlPiece += u'</tr>'

        for line in data:
                htmlPiece += u'<tr>'
                for fl in line:
                    htmlPiece += u'<td>'
                    htmlPiece += str(fl)
                    htmlPiece += u'</td>'
                htmlPiece += u'</tr>'
        htmlPiece += u'\n'
        htmlPiece += u'</tbody>'
        htmlPiece += u'\n'
        htmlPiece += u'</table>'
        htmlPiece += u'</div>'

        if self.htmlContainer:
            self.htmlContainer += u'\n'
        self.htmlContainer += location
        self.htmlContainer += u'\n'
        self.htmlContainer += ChartDataHandler.htmlJumper
        self.htmlContainer += u'\n'
        self.htmlContainer += htmlPiece

        self.id += 1

    def generateBarChartData(self, reportName, displayName, custom=False):
        logging.info('ChartDataHandler.generateBarChartData')

        if custom:
            if reportName not in self.reportData['custom_reports']:
                return
            else:
                report  = self.reportData['custom_reports'][reportName]
        else:
            if reportName not in self.reportData:
                return
            else:
                report  = self.reportData[reportName]

        catSet  = []
        dataSet = []
        if not report:
            report = []

        logging.debug("report_name:%s, Debug_Report: %s", reportName, report)
        for name,value in report:
            catSet.append(name.replace("'", "").replace("\\", "\\\\"))
            dataSet.append(value)
        if len(catSet) < 5:
            for i in xrange( 5 - len(catSet) ):
                catSet.append('')
                dataSet.append('')
        if displayName:
            self._createSelector(displayName)
        container = self._createContainer(250 + 10*len(catSet))
        ret = {
                'chart': {
                    'renderTo': container,
                    'type': 'bar',
                    'marginRight': 80,
                    'marginLeft': 180
                       },
                'credits': {
                    'enabled': False
                        },
                'title': {
                    'text': displayName
                       },
                'xAxis': {
                    'categories': catSet
                       },
                'tooltip': {
                        },
                'yAxis': {
                    'title': {
                    'text': ' '
                             }
                       },
                'plotOptions': {
                    'bar' :{
                        'shadow': False,
                        'dataLabels':{
                             'enabled': True
                                   }
                        }
                    },
                'series': [{
                    'name': displayName,
                    'data': dataSet
                    }],
                'legend': {
                    'enabled': False
                }
              }
        return ret

    def generatePieChartData(self, reportName, displayName, total = 0, custom=False):
        logging.debug('chartDataHandler.generatePieChartData')

        if custom:
            if reportName not in self.reportData['custom_reports']:
                return
            else:
                report  = self.reportData['custom_reports'][reportName]
        else:
            if reportName not in self.reportData:
                return
            else:
                report  = self.reportData[reportName]

        dataSet = []
        if not report:
            report = []

        logging.debug("report_name:%s, Debug_Report: %s", reportName, report)
        for name,value in report:
            dataSet.append([name.replace("'", ""),value])
            if total > 0:
                total -= int(value)

        if total > 0:
            dataSet.append(['Other',total])

        if displayName:
            self._createSelector(displayName)
        container = self._createContainer()
        ret = {
            'chart': {
                'renderTo': container,
                'type': 'pie',
                'plotBackgroundColor': None,
                'plotBorderWidth': None,
                'plotShadow': False
            },
            'credits': {
                    'enabled': False
                        },
            'title': {
                'text': displayName
            },
            'tooltip': {
            },
            'legend':  {
                'enabled': True
            },
            'plotOptions': {
                'pie': {
                    'allowPointSelect': True,
                    'cursor': 'pointer',
                    'dataLabels': {
                        'enabled': True,
                        'color': '#000000',
                        'connectorColor': '#000000',
                        'formatter' : {}
                    },
                    'showInLegend': True,
                    'shadow': False
                }
            },
            'series': [{
                'type': 'pie',
                'name': displayName,
                'data': dataSet
            }]
        }
        return ret

    def generateLineChartData(self, reportName, displayName, custom=False):
        logging.info('ChartDataHandler.generateLineChartData')

        if custom:
            if reportName not in self.reportData['custom_reports']:
                return
            else:
                report  = self.reportData['custom_reports'][reportName]
        else:
            if reportName not in self.reportData:
                return
            else:
                report  = self.reportData[reportName]

        if not report:
            report = {}
        self._getLineTimePoint()

        series = []
        dataPoint = self._getLineDataPoint(report)
        seItem = {
            'name':displayName,
            'data':dataPoint
        }
        series.append(seItem)

        logging.info('ChartDataHandler.generateLineChartData')
        if displayName:
            self._createSelector(displayName)
        container = self._createContainer()
        ret = {
            'chart': {
                'renderTo': container,
                'type': 'spline',
                'marginRight': 130,
                'marginLeft': 130
            },
            'credits': {
                    'enabled': False
                        },
            'title': {
                'text': displayName
            },
            'xAxis': {
                'categories': self.lineTimeszSet,
                'tickInterval': None,
                'labels': {
                    'align': 'right',
                    'rotation': -45
                }
            },
            'yAxis': {
                'min': 0,
                'title': {
                    'text': ''
                },
                'plotLines': [{
                    'value': 0,
                    'width': 1,
                    'color': '#808080'
                }]
            },
            'plotOptions': {
                'spline': {
                    'lineWidth': 2,
                    'states': {
                        'hover': {
                            'lineWidth': 3
                        }
                    },
                    'marker': {
                        'enabled': True,
                        'states': {
                            'hover': {
                                'enabled': True,
                                'symbol': 'circle',
                                'radius': 3,
                                'lineWidth': 2
                            }
                        }
                    },
                    'enableMouseTracking': False,
                    'shadow': False
                }
            },
            'tooltip': {
            },
            'credits': {
                'enabled': False
            },
            'series': series,
            'legend': {
                'enabled': False
            }
        }
        return ret


    def _getDayTs(self, ts):
        '''Convert the timestamp to the begin of the day'''
        return int(time.mktime(datetime.date.fromtimestamp(int(ts)).timetuple()))

    def _getLineDataPoint(self, reportLineData):
        logging.debug('ChartDataHandler._getLineDataPoint')

        dataSet ={}
        for tm,value in reportLineData.items():
            tm = str(self._getDayTs(tm))
            if dataSet.has_key(tm):
                dataSet[tm] += int(value)
            else:
                dataSet[tm] = int(value)

        valueSet =[]

        for tm in self.lineTimePoints:
            if tm in dataSet:
                valueSet.append(dataSet[tm])
            else:
                valueSet.append(0)
        return valueSet

    def _getLineTimePoint(self):
        logging.debug('ChartDataHandler._getLineTimePoint')

        if self.lineTimePoints:
            return

        timeFrom = self.info.timeFrom
        timeTo   = self.info.timeTo

        endDate   = datetime.datetime.fromtimestamp(timeTo)
        beginDate = datetime.datetime.fromtimestamp(timeFrom)
        beginDate = datetime.datetime.date(beginDate)

        tmDate = beginDate
        deltatime = datetime.timedelta(days=1)
        while time.mktime(tmDate.timetuple()) <= timeTo:
            self.lineTimePoints.append(str(int(time.mktime(tmDate.timetuple()))))
            self.lineTimeszSet.append(tmDate.strftime('%Y/%m/%d'))
            tmDate += deltatime

    def _createContainer(self, count=300):
        logging.debug('ChartDataHandler._createContainer')
        container = u'container%d'%(self.id)
        if self.id == 0 or self.id == 1:
            htmlPiece = u'<div id="%s" style="width: 700px; margin:0 auto;"></div>'%container
        else:
            htmlPiece = u'<div id="%s" style="width: 600px; margin:0 auto;"></div>'%(container)

        location = u'''<a name="loc%d" ></a>'''%(self.id)
        if self.htmlContainer:
            self.htmlContainer += u'\n'
        self.htmlContainer += location
        self.htmlContainer += u'\n'
        self.htmlContainer += ChartDataHandler.htmlJumper
        self.htmlContainer += u'\n'
        self.htmlContainer += htmlPiece


        self.id += 1
        return container

    def _createSelector(self, name):
        logging.debug('ChartDataHandler._createSelector')
        option = u'''<option value ="#loc%d">%s</option>'''%(self.id,name)
        if self.htmlSelector:
            self.htmlSelector += u'\n'
        self.htmlSelector += option

    def getHtmlSelector(self):
        logging.debug('ChartDataHandler.getHtmlSelector')
        return self.htmlSelector

    def getChartData(self):
        logging.debug('ChartDataHandler.getChartData')
        return json.dumps(self.chartData, ensure_ascii=False)

    def getHtmlContainer(self):
        logging.debug('ChartDataHandler.getHtmlContainer')
        return self.htmlContainer

    def getReportTitle(self):
        logging.debug('ChartDataHandler.getReportTitle')
        return self.info.reportTitle

    def getGeneratedTime(self):
        logging.debug('ChartDataHandler.getGeneratedTime')
        return time.strftime('%Y-%m-%d %H:%M:%S %Z', time.localtime(self.info.taskTime))

    def getTimeFrom(self):
        logging.debug('ChartDataHandler.GetTimeFrom')
        return time.strftime('%Y-%m-%d %H:%M:%S %Z', time.localtime(self.info.timeFrom))

    def getTimeTo(self):
        logging.debug('ChartDataHandler.getTimeTo')
        return time.strftime('%Y-%m-%d %H:%M:%S %Z', time.localtime(self.info.timeTo))

class HTMLReportGenerator(ChartDataHandler):

    #class constants
    HTML_TEMPLET_FILE = 'html/reportTemplet.html'
    PDF_TEMPLET_FILE  = 'html/reportTempletForPDF.html'

    chartDataTag       = u'$$chartData$$'
    chartConatinersTag = u'$$chartConatiners$$'
    reportTitleTag     = u'$$reportTitle$$'
    generatedTimeTag   = u'$$generatedTime$$'
    timeFromTag        = u'$$timeFrom$$'
    timeToTag          = u'$$timeTo$$'
    selectorTag        = u'$$selector'

    def __init__(self, params, info, reportData):
        logging.debug('HTMLReportGenerator.__init__:')
        ChartDataHandler.__init__(self, params, info, reportData)

    def buildReport(self, reportFileName):
        '''Generate two HTML files: one for HTML report, another will be converted to PDF format.'''
        logging.debug('HTMLReportGenerator.buildReport:')

        self.buildChartData()

        ###################################################################################
        #generate html report report

        content = u''
        template_file_path = os.path.join(currentDir, HTMLReportGenerator.HTML_TEMPLET_FILE)

        fileObj = codecs.open(template_file_path, 'r', 'utf-8')
        content = fileObj.read()
        fileObj.close()

        content = content.replace(HTMLReportGenerator.chartConatinersTag, self.getHtmlContainer())
        content = content.replace(HTMLReportGenerator.reportTitleTag,     self.getReportTitle())
        content = content.replace(HTMLReportGenerator.generatedTimeTag,   self.getGeneratedTime())
        content = content.replace(HTMLReportGenerator.timeFromTag,        self.getTimeFrom())
        content = content.replace(HTMLReportGenerator.timeToTag,          self.getTimeTo())
        content = content.replace(HTMLReportGenerator.selectorTag,        self.getHtmlSelector())
        #replace chart data lastly
        content = content.replace(HTMLReportGenerator.chartDataTag, self.getChartData())

        fileName = reportFileName + '.html'
        reportFilePath = os.path.join(ReportConfig.REPORT_TEMP_LOCATION, fileName)
        logging.debug("generate html file: (%s)" % reportFilePath)
        reportFileObj = codecs.open(reportFilePath, 'w', 'utf-8')
        reportFileObj.write(content)
        reportFileObj.close()

        ###################################################################################
        #generate html report for pdf report

        contentForPdf = u''
        pdf_templet_file_path = os.path.join(currentDir, HTMLReportGenerator.PDF_TEMPLET_FILE)

        fileObjForPdf = codecs.open(pdf_templet_file_path, 'r', 'utf-8')
        contentForPdf = fileObjForPdf.read()
        fileObjForPdf.close()

        contentForPdf = contentForPdf.replace(HTMLReportGenerator.chartConatinersTag, self.getHtmlContainer())
        contentForPdf = contentForPdf.replace(ChartDataHandler.htmlJumper, u'')
        contentForPdf = contentForPdf.replace(HTMLReportGenerator.reportTitleTag, self.getReportTitle())
        contentForPdf = contentForPdf.replace(HTMLReportGenerator.generatedTimeTag, self.getGeneratedTime())
        contentForPdf = contentForPdf.replace(HTMLReportGenerator.timeFromTag, self.getTimeFrom())
        contentForPdf = contentForPdf.replace(HTMLReportGenerator.timeToTag, self.getTimeTo())
        #replace chart data lastly
        contentForPdf = contentForPdf.replace(HTMLReportGenerator.chartDataTag, self.getChartData())

        fileNameForPdf = reportFileName + '-2.html'
        reportFileForPdfPath = os.path.join(ReportConfig.REPORT_TEMP_LOCATION, fileNameForPdf)
        logging.debug("generate html file: (%s)" % reportFileForPdfPath)
        reportFileObjForPdf = codecs.open(reportFileForPdfPath, 'w', 'utf-8')
        reportFileObjForPdf.write(contentForPdf)
        reportFileObjForPdf.close()

        reportFileHTML    = reportFilePath
        reportFileForPDF  = reportFileForPdfPath
        return (reportFileHTML, reportFileForPDF)

class CSVReportGenerator():
    def __init__(self, params, info, reportData):
        self.params         = params
        self.info           = info
        self.reportData     = reportData

    def buildReport(self, reportFileName):
        logging.debug('CSVReportGenerator.buildReport')
        CVSData = self.buildCSVData()
        fileName = reportFileName + '.csv'
        reportFilePath = os.path.join(ReportConfig.REPORT_TEMP_LOCATION, fileName)
        logging.debug("generate CSV file: (%s)" % reportFilePath)
        reportFileObj = codecs.open(reportFilePath, 'w', 'utf-8')
        reportFileObj.write(CVSData)
        reportFileObj.close()

        return reportFilePath

    def buildCSVData(self):
        logging.debug('CSVReportGenerator.buildChartData')

        CSVData = u'\uFEFF'
        CSVData += 'InterScan Web Security Virtual Appliance\n\n'
        CSVData += 'Report Name,%s\n' % self.info.reportTitle
        CSVData += 'Generated On,%s\n' % util.timestamp2string(time.time())
        CSVData += 'Data Range,%s-%s\n' % (util.timestamp2string(self.info.timeFrom), util.timestamp2string(self.info.timeTo))
        for item in self.params:
            data = self.generateCSVData(item.favID if item.custom else item.reportName, item.displayName, item.chartType, item.custom, item.viewChart)
            CSVData= ''.join((CSVData, '\n', data))
        logging.debug('cvs data: %s', CSVData)
        return CSVData

    def generateCSVData(self, reportName, displayName, itemType, custom, viewChart):
        logging.info('ChartDataHandler.generateCSVData: name(%s), chartType(%s)', displayName, itemType)

        if custom:
            if reportName not in self.reportData['custom_reports']:
                return
            else:
                report  = self.reportData['custom_reports'][reportName]
        else:
            if reportName not in self.reportData:
                return
            else:
                report  = self.reportData[reportName]

        data = displayName + '\n';
        if custom:
                #generate facet table
                for k in report:
                    if k == 'CLF_Fields':
                        continue
                    if report.has_key('CLF_Fields'):
                        data = ''.join((data, ','.join((report['CLF_Fields'][k], '')) + '\n'))
                    else:
                        data = ''.join((data, ','.join((k, '')) + '\n'))
                    for key,value in report[k]:
                        key = key.replace('<font color="#0080ff">','').replace('<u>','').replace('</u>','').replace('</font>','').replace('<b>','').replace('</b>','')
                        data = ''.join((data, ','.join((key, str(value))) + '\n'))

                #generate chart
                if itemType == chartType['line']:
                    #time chart
                    report = self.reportData['custom_reports']['/'.join([reportName, 'timeChart'])]
                    keyList = []
                    for key in report:
                        keyList.append(key)
                    keyList.sort()
                    for key in keyList:
                        data = ''.join((data, ','.join((util.timestamp2string(key), str(report[key]))) + '\n'))
                else:
                    if viewChart:
                        reportNm = '/'.join([reportName, viewChart])
                        report = self.reportData['custom_reports'][reportNm]
                        if itemType == chartType['bar']:
                            for key, value in report:
                                data = ''.join((data, ','.join((key, str(value))) + '\n'))
                        elif itemType == chartType['pie']:
                            for key, value in report:
                                data = ''.join((data, ','.join((key, str(value))) + '\n'))

                #get raw log
                rawLogs = self.reportData['custom_reports']['/'.join((reportName, 'rawLogs'))]
                if rawLogs:
                    for row in rawLogs:
                        data = ''.join((data, ','.join(row) + '\n'))
        else:
            if itemType == chartType['bar']:
               for key, value in report:
                    data = ''.join((data, ','.join((key, str(value))) + '\n'))
            elif itemType == chartType['pie']:
                for key, value in report:
                    data = ''.join((data, ','.join((key, str(value))) + '\n'))
            elif itemType == chartType['line']:
                keyList = []
                for key in report:
                    keyList.append(key)
                keyList.sort()
                for key in keyList:
                    data = ''.join((data, ','.join((util.timestamp2string(key), str(report[key]))) + '\n'))
            elif itemType == chartType['table']:
                for key,value in report:
                    key = key.replace('<font color="#0080ff">','').replace('<u>','').replace('</u>','').replace('</font>','').replace('<b>','').replace('</b>','')
                    data = ''.join((data, ','.join((key, str(value))) + '\n'))

        logging.debug('generate csv data result: %s', data)
        if data:
            return data

class PDFReportGenerator():
    @staticmethod
    def buildReport(htmlReportFile, reportFileName):
        logging.debug('pdfReportGenerator.buildReport')
        outputFile = os.path.join(ReportConfig.REPORT_TEMP_LOCATION, reportFileName + '.pdf')

        pdfCmd = currentDir + '/wkhtmltopdf-amd640.9' + \
                 ' --redirect-delay 3000 --footer-spacing 3 --header-spacing 2 ' + \
                 htmlReportFile + ' ' + outputFile
        logging.debug(pdfCmd)
        os.system(pdfCmd)

        #if os.path.isfile(htmlReportFile):
        #    os.remove(htmlReportFile)
        if not os.path.isfile(outputFile):
            logging.error('PDFReportGenerator.buildReport: Error, fail to get file(%s)', outputFile)
            outputFile = None

        return outputFile

if __name__ == '__main__':
    pass
