
is_interactive()
{
	if [ -z "$CLI_NONE_INTERACTIVE" ] ; then
		return 1
	else
		return 0
	fi
}

press_anyKey() 
{
	if [ -z "$CLI_NONE_INTERACTIVE" ] ; then
		read -s -n 1 -p "Press any key to continue..."
		echo
	fi
}

pipe_more()
{
	if [ -z "$CLI_NONE_INTERACTIVE" ] ; then
		$@ | /bin/more
	else
		$@
	fi
}

more_file()
{
	if [ -z "$CLI_NONE_INTERACTIVE" ] ; then
		/bin/more $*
	else
		/bin/cat $*
	fi
}

confirm()
{
	prompt=$1
	if [ -z "$CLI_NONE_INTERACTIVE" ] ; then
		if [ -z "$prompt" ] ; then
			prompt="Do you want to proceed? (y/n)"
		fi
		read -n 1 -p "$prompt" ANS
		if [ "$ANS" = "y" ] ; then
			return 0
		else
			return 1
		fi
	else
		return 0
	fi
}

interactive_echo()
{
	if [ -z "$CLI_NONE_INTERACTIVE" ] ; then
		echo $*	
	fi
}

symlink_basename()
{
	path=$1
	ls=`ls -l $path`
	basename=`basename "$ls"`
	RET=$basename
}

change_db_pwd()
{
            echo  -n "Please enter the new db password: "
            read -r -s new_password
            echo
            echo -n "Please confirm the new db password: "
            read -r -s confirm_password
            echo

            password_len=`expr length "$new_password"`
            if [ $password_len -lt 6 ]; then
                echo "db password not changed: Password length is less than 6. "
                exit
            fi

            if [ $password_len -gt 128 ]; then
                echo "db password not changed: Password length cannot be more than 128."
                exit
            fi

            echo "$new_password" | grep " " >/dev/null
            if [ $? -eq 0 ]; then
                echo "Password cannot contain spaces or tabs. Please type a valid password."
                exit
            fi

            echo "$new_password" | grep "	" >/dev/null
            if [ $? -eq 0 ]; then
                echo "Password cannot contain spaces or tabs. Please type a valid password."
                exit
            fi

            echo "$new_password" | grep ";" >/dev/null
            if [ $? -eq 0 ]; then
                echo "Password cannot contain semicolons. Please type a valid password."
                exit
            fi
            

            if [ "$new_password" = "$confirm_password" ]; then

                su - iscan -c "/usr/iwss/PostgreSQL/bin/psql -h localhost -p 5432 -Usa iwss -c '\d'" >/dev/null
                if [ $? -ne 0 ]; then
                    echo -e "\nError: Cannot connect to database.\n"
                    exit
                fi

                temp_password=${new_password//\\/\\\\}
                temp_password=${temp_password//\'/\\\'}

su - iscan -c "/usr/iwss/PostgreSQL/bin/psql -h localhost -p 5432 -Usa template1" <<end_of_file
alter user sa with password '$temp_password';
alter user iscan with password '$temp_password';
end_of_file

                local remote_db=`iniconfig -r -f /etc/iscan/intscan.ini -s Database -o remote_db`
                local arm_db=`iniconfig -r -f /etc/iscan/intscan.ini -s Database -o ARM_DB`

                local change_log_setting="yes"
                if [ "$remote_db" = "yes" -o "$arm_db" = "yes" ]; then
                    change_log_setting="no"
                fi

                #temp_password=${new_password//\;/\\\;}
                #echo $temp_password
                ENCPWD_SA=`/usr/iwss/bin/encpw "$new_password"`

                #sed -i "s/\(Password=\).*/\1$ENCPWD_SA/" /etc/iscan/intscan.ini

                iniconfig -w -f /etc/iscan/intscan.ini -s Database -o Password -n "$ENCPWD_SA"

                if [ "$change_log_setting" = "yes" ]; then
                    iniconfig -w -f /etc/iscan/intscan.ini -s Database_Log -o Password -n "$ENCPWD_SA"
                fi

                #sed -i "/^Password/d" /etc/iscan/odbc.ini
                #echo "Password            = $new_password" >> /etc/iscan/odbc.ini
                #sed -i "/^$/d" /etc/iscan/odbc.ini

                iniconfig -w -f /etc/iscan/odbc.ini -s IWSS -o Password -n "$new_password"
                if [ "$change_log_setting" = "yes" ]; then
                    iniconfig -w -f /etc/iscan/odbc.ini -s IWSS_LOG -o Password -n "$new_password"
                fi

                temp_pgpass="/tmp/cli_temp_pgpass"

                PGPASS_FILE="/var/iwss/.pgpass"

                cat $PGPASS_FILE | grep -E -v "^localhost" >$temp_pgpass 
                cat $temp_pgpass >$PGPASS_FILE
                echo "localhost:5432:*:sa:$new_password" >> $PGPASS_FILE
                echo "localhost:5432:*:iscan:$new_password" >> $PGPASS_FILE

                chown iscan $PGPASS_FILE
                chgrp iscan $PGPASS_FILE
                chmod 600 $PGPASS_FILE

                PGPASS_FILE="/root/.pgpass"

                cat $PGPASS_FILE | grep -E -v "^localhost" >$temp_pgpass 
                cat $temp_pgpass >$PGPASS_FILE
                echo "localhost:5432:*:sa:$new_password" >> $PGPASS_FILE
                echo "localhost:5432:*:iscan:$new_password" >> $PGPASS_FILE

                chown root $PGPASS_FILE
                chgrp root $PGPASS_FILE
                chmod 600 $PGPASS_FILE

                PGPASS_FILE="/home/enable/.pgpass"
                cat $PGPASS_FILE | grep -E -v "^localhost" >$temp_pgpass 
                cat $temp_pgpass >$PGPASS_FILE
                echo "localhost:5432:iwss:sa:$new_password" >> $PGPASS_FILE

                chown enable $PGPASS_FILE
                chgrp root $PGPASS_FILE
                chmod 600 $PGPASS_FILE
                su - iscan -c "/usr/iwss/dbctl.sh reload"

                rm -f $temp_pgpass

                /etc/iscan/S99IScanHttpd restart
            else
                echo "db password not changed: New password and confirm password mismatch."
            fi
}

check_port_list()
{
    ports_list=$1
    exception_ports=$2

    ports_list=`echo "$ports_list" | sed 's/;/ /g'`
        
    for port in $ports_list ; do

        # check if the port is a interest port 
        re='(^PORT;)|(;PORT;)|(;PORT$)|(^PORT$)'
        re=`echo $re | sed  -e "s/PORT/$port/g"`

        echo $exception_ports  | /bin/egrep -q "$re"
        if [ "$?" = "0" ]; then
            continue
        fi

        sudo /usr/sbin/lsof -i:${port} | grep "LISTEN"
        if [  "$?" = "0" ]; then
            echo -e "Error: ${port} has been used by another daemon. You can use the \"show network connections listening tcp\" command to display port-daemon mappings."
            echo
            return 1
        fi
    done

    return 0
}

post_update_timezone()
{
    count=`ps -ef | grep "org.apache.catalina.startup.Bootstrap" | grep -v grep | wc -l`

    if [ "$count" -ne 0 ] && [ "XX${CLI_NONE_INTERACTIVE}" != "XX1" ] ; then
        sudo /var/iwss/S99IScanHttpd stop > /dev/null; sleep 3 ;sudo /var/iwss/S99IScanHttpd start > /dev/null
    fi

    sudo /sbin/service crond restart > /dev/null
}

# used for ARM register
disable_database_command()
{
    UIHELPER="/usr/iwss/AdminUI/uihelper"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN/g' /etc/clish/configure_module.xml"
    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_END-->$/COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/configure_module.xml"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN/g' /etc/clish/show_module.xml"
    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_END-->$/COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/show_module.xml"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN/g' /etc/clish/start_service.xml"
    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_END-->$/COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/start_service.xml"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN/g' /etc/clish/start_task.xml"
    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_END-->$/COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/start_task.xml"

    return 0
}

enable_database_command()
{
    UIHELPER="/usr/iwss/AdminUI/uihelper"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->/g' /etc/clish/configure_module.xml"
    $UIHELPER "sed -i 's/^COMMAND_FOR_LOCAL_DATABASE_END-->$/<!--COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/configure_module.xml"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->/g' /etc/clish/show_module.xml"
    $UIHELPER "sed -i 's/^COMMAND_FOR_LOCAL_DATABASE_END-->$/<!--COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/show_module.xml"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->/g' /etc/clish/start_service.xml"
    $UIHELPER "sed -i 's/^COMMAND_FOR_LOCAL_DATABASE_END-->$/<!--COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/start_service.xml"

    $UIHELPER "sed -i 's/^<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN$/<!--COMMAND_FOR_LOCAL_DATABASE_BEGIN-->/g' /etc/clish/start_task.xml"
    $UIHELPER "sed -i 's/^COMMAND_FOR_LOCAL_DATABASE_END-->$/<!--COMMAND_FOR_LOCAL_DATABASE_END-->/g' /etc/clish/start_task.xml"
    return 0
}

# confgiure route [interface, network_id, mask, gateway]
configure_interface_route_v4()
{
    if [ $# -ne 4 ]; then
        echo "ERROR: Invalid parameter."
        return 1
    fi

    local device=$1
    local address=$2
    local netmask=$3
    local gateway=$4

    local index=0

    local route_cfg="/etc/sysconfig/network-scripts/route-$device"

    if [ -e $route_cfg ]; then
        index=`cat $route_cfg | grep '^ADDRESS*' | wc -l`

        # check if the route has been existed 
        local cnt=0
        while [ $cnt -lt $index ]; do

            existed_addr=`cat $route_cfg | grep "^ADDRESS$cnt" | cut -d'=' -f2`
            existed_mask=`cat $route_cfg | grep "^NETMASK$cnt" | cut -d'=' -f2`
            if [ "$existed_addr" = "$address" -a "$existed_mask" = "$netmask" ]; then
                echo "Configured route already exists in the current route table."
                echo ""
                return 1
            fi

            cnt=$(($cnt +1))
        done

    fi

    sudo echo "ADDRESS$index=$address" >>$route_cfg
    sudo echo "NETMASK$index=$netmask" >>$route_cfg
    sudo echo "GATEWAY$index=$gateway" >>$route_cfg
    sudo echo "TMOPTION$index=0" >>$route_cfg

    return 0
}

configure_interface_route_v6()
{
    if [ $# -ne 4 ]; then
        echo "ERROR: Invalid parameter."
        return 1
    fi

    local device=$1
    local address=$2
    local prefix=$3
    local gateway=$4
    local route6_cfg="/etc/sysconfig/network-scripts/route6-$device"
    local tmpstr="$address/$prefix via $gateway"

    if [ -e $route6_cfg ]; then
	if cat $route6_cfg | grep '^[[:space:]]*[0-9a-fA-F:]*/[0-9]*' | grep -q "$tmpstr" ; then
            echo "Configured route already exists in the current route table."
            return 1
	fi
    fi

    sudo echo "$tmpstr" >> $route6_cfg
    return 0
}

configure_interface_route()
{
    if [ $# -le 1 ]; then
        echo "ERROR: Invalid parameter."
        return 1
    fi
    local proto="$1"
    shift

    case $proto in
	ipv4)
	    configure_interface_route_v4 $@
	    ;;
	ipv6)
	    configure_interface_route_v6 $@
	    ;;
	*)
	    echo "ERROR: Invalid IP proto."
	    return 1
	    ;;
    esac
}

delete_interface_route_v4()
{
    if [ $# -ne 4 ]; then
        echo -e "\nError: Invalid parameter.\n"
        return 1
    fi

    local device=$1
    local address=$2
    local netmask=$3
    local gateway=$4
    local route_cfg="/etc/sysconfig/network-scripts/route-$device"

    if ! [ -e $route_cfg ]; then
        echo -e "\nError: This route does not exist in current route table.\n"
        return 1
    fi

    # check if the route has been existed 
    local cnt=0
    local new_index=0
    local find_route="no"
    local tmp_route_cfg="/tmp/clish.route.temp"
    local index=`cat $route_cfg | grep '^ADDRESS*' | wc -l`
    
    echo "#`date`: Generate by IWSVA network scripts" >$tmp_route_cfg

    while [ $cnt -lt $index ]; do
        
        existed_addr=`cat $route_cfg | grep "^ADDRESS$cnt" | cut -d'=' -f2`
        existed_mask=`cat $route_cfg | grep "^NETMASK$cnt" | cut -d'=' -f2`
        existed_gw=`cat $route_cfg | grep "^GATEWAY$cnt" | cut -d'=' -f2`
        existed_opt=`cat $route_cfg | grep "^TMOPTION$cnt" | cut -d'=' -f2`

        if [ "$existed_addr" != "$address" -o "$existed_mask" != "$netmask" -o "$existed_gw" != "$gateway" ]; then

            echo "ADDRESS$new_index=$existed_addr" >>$tmp_route_cfg
            echo "NETMASK$new_index=$existed_mask" >>$tmp_route_cfg
            echo "GATEWAY$new_index=$existed_gw" >>$tmp_route_cfg
            echo "TMOPTION$new_index=$existed_opt" >>$tmp_route_cfg

            new_index=$(($new_index +1))
        else
            find_route="yes"
        fi

        cnt=$(($cnt +1))
    done

    if [ $find_route != "yes" ]; then
        echo -e "\nError: This route does not exist in current route table.\n"
        return 1
    fi

    sudo mv -f $tmp_route_cfg $route_cfg
    return 0
}

delete_interface_route_v6()
{
    if [ $# -ne 4 ]; then
        echo -e "\nError: Invalid parameter.\n"
        return 1
    fi

    local device=$1
    local address=$2
    local prefix=$3
    local gateway=$4
    local route6_cfg="/etc/sysconfig/network-scripts/route6-$device"
    local tmpstr="$address/$prefix via $gateway"
    local regtmp="^[[:space:]]*`echo $tmpstr | sed 's%[\:\/]%\\\\&%g' | sed 's%\ %\[\[\:space\:\]\]%g'`"

    if ! [ -e $route6_cfg ]; then
        echo -e "\nError: This ipv6 route does not exist in current route table.\n"
        return 1
    fi

    if ! `cat $route6_cfg | grep -q "$tmpstr"` ; then
        echo -e "\nError: This ipv6 route does not exist in current route table.\n"
        return 1
    fi

    sudo sed -i "/$regtmp/d" $route6_cfg
}

delete_interface_route()
{
    if [ $# -le 1 ]; then
        echo "ERROR: Invalid parameter."
        return 1
    fi
    local proto="$1"
    shift

    case $proto in
	ipv4)
	    delete_interface_route_v4 $@
	    ;;
	ipv6)
	    delete_interface_route_v6 $@
	    ;;
	*)
	    echo "ERROR: Invalid IP proto."
	    return 1
	    ;;
    esac
}

update_timezone()
{
    if [ $# -eq 2 ]; then
        timezone=$1/$2
    elif [ $# -eq 3 ]; then
        timezone=$1/$2/$3
    else
        return
    fi
    
    sudo /bin/cp --remove-destination /usr/share/zoneinfo/${timezone} /etc/localtime
    sudo /usr/bin/rwconfig -ct -d '=' -f /etc/sysconfig/clock -s ZONE -n "${timezone}\""
    sudo /sbin/hwclock --systohc

    post_update_timezone
}

update_keyboard()
{
    UIHELPER="/usr/iwss/AdminUI/uihelper"
    sudo sed -i "/KEYTABLE/s/\".*\"/\"$1\"/" /etc/sysconfig/keyboard

    if [ -x /bin/loadkeys ]; then
        KEYTABLE=
        KEYMAP=
        if [ -f /etc/sysconfig/console/default.kmap ]; then
            KEYMAP=/etc/sysconfig/console/default.kmap
        else
            if [ -f /etc/sysconfig/keyboard ]; then
                . /etc/sysconfig/keyboard
            fi
            if [ -n "$KEYTABLE" -a -d "/lib/kbd/keymaps" ]; then
                 KEYMAP="$KEYTABLE.map"
            fi
        fi

        if [ -n "$KEYMAP" ]; then
            if [ -n "$KEYTABLE" ]; then
                echo -n $"Loading default keymap ($KEYTABLE): "
            else
                echo -n $"Loading default keymap: "
            fi
            LOADKEYS=loadkeys
            if [ "${LANG}" != "${LANG%%.UTF-8}" -o "${LANG}" != "${LANG%%.utf8}" ]; then
                LOADKEYS="loadkeys -u"
            fi
            $UIHELPER ". /etc/init.d/functions; \
                $LOADKEYS $KEYMAP < /dev/tty0 > /dev/tty0 2>/dev/null && \
                success $\"Loading default keymap\" || failure $\"Loading default keymap\""
            echo
        fi
    fi
}

findNewDisk()
{
    awk -F"[ :]" '
        BEGIN {i=1}

        /^Disk/ {
            if (i == 0) newDisks[disk]=1
            disk=$2
            i=0
        }

        { if (index($1, disk) > 0) ++i }

        END {
            if (i == 0) newDisks[disk]=1
            for (disk in newDisks) printf("%s ", disk)
            printf("\n");
        }
    ' $TMP_DISK_INFO
}

init_disk()
{
    newDisk=$1
    size=`awk -F"[: ]" -v DISKNAME=$newDisk '/bytes/ {if($2 == DISKNAME) {getline;print $5}}' $TMP_DISK_INFO`

sudo /sbin/fdisk $newDisk <<EOF >>/dev/null 2>&1
n
p
1
1
$size
t
8e
w
EOF

    if [ $? != 0 ]; then
        echo "Failed to add a new partition on $newDisk."
        return $ERR_ADD_NEW_PARTITION
    fi

    return 0
}

add_new_disk()
{
    ERR_NO_NEW_DISK=1
    ERR_ADD_NEW_PARTITION=2
    ERR_CREATE_PV=3
    ERR_EXTEND_VG=4
    ERR_EXTEND_LV=5
    ERR_EXTEND_FS=6

    TMP_DISK_INFO=/tmp/.disk.log

    #begin:modify because disk info change from "/dev/dm-*" to "/dev/mapper/*"
    #sudo /sbin/fdisk -l 2>/dev/null | grep -v "Disk /dev/dm-" | grep -v "Disk identifier" | grep . >$TMP_DISK_INFO  
    sudo /sbin/fdisk -l 2>/dev/null | grep -v "Disk /dev/mapper" | grep -v "Disk identifier" | grep . >$TMP_DISK_INFO
    #end modify

    DISKS=$(findNewDisk)
    if [ -z "$DISKS" ];then
        echo "No new disk was found."
        return $ERR_NO_NEW_DISK
    fi

    for disk in $DISKS; do
        init_disk $disk
        #begin: add status checking for init_disk
        if [ $? != 0 ]; then
            echo "Failed to init disk ${disk}."
            return $ERR_ADD_NEW_PARTITION
        fi
        #end add

        sudo /sbin/pvcreate ${disk}1 >/dev/null
        if [ $? != 0 ]; then
            echo "Failed to create a PV on ${disk}1."
            return $ERR_CREATE_PV
        fi

        sudo /sbin/vgextend IWSVA ${disk}1 >/dev/null
        if [ $? != 0 ]; then
            echo "Failed to extend the VG IWSVA."
            return $ERR_EXTEND_VG
        fi
    done

    freeSize=`sudo /sbin/vgdisplay IWSVA | awk '/Free/ {printf("%d", $7)}'` >/dev/null
    sudo /sbin/lvextend -l +100%FREE /dev/IWSVA/app_data >/dev/null
    if [ $? != 0 ]; then
        echo "Failed to extend the LV /dev/IWSVA/app_data."
        return $ERR_EXTEND_LV
    fi

    sudo /sbin/resize2fs /dev/IWSVA/app_data >/dev/null 2>&1
    if [ $? != 0 ]; then
        echo "Failed to extend the file system on /dev/IWSVA/app_data LV."
        return $ERR_EXTEND_FS
    fi

    rm -f $TMP_DISK_INFO
    echo "Add the new disk $DISKS successfully."
    return 0
}

